@@ -1,4 +1,5 @@
-"""
+"""All-in-one CrowdStrike Falcon OAuth2 API harness
+
  _______                        __ _______ __        __ __
 |   _   .----.-----.--.--.--.--|  |   _   |  |_.----|__|  |--.-----.
 |.  1___|   _|  _  |  |  |  |  _  |   1___|   _|   _|  |    <|  -__|
@@ -9,8 +10,6 @@
 
 OAuth2 API - Customer SDK
 
-api_complete - All-in-one CrowdStrike Falcon OAuth2 API harness
-
 This is free and unencumbered software released into the public domain.
 
 Anyone is free to copy, modify, publish, use, compile, sell, or
@@ -38,12 +37,13 @@ For more information, please refer to <https://unlicense.org>
 """
 import time
 from ._util import _ALLOWED_METHODS
-from ._util import perform_request, generate_b64cred, generate_error_result, calc_url_from_args
+from ._util import perform_request, generate_b64cred, generate_error_result
+from ._util import confirm_base_url, calc_url_from_args
 from ._endpoint import api_endpoints
 
 
 class APIHarness:
-    """ This one does it all. It's like the One Ring with significantly fewer orcs. """
+    """This one does it all. It's like the One Ring with significantly fewer orcs."""
     # pylint: disable=too-many-instance-attributes
 
     TOKEN_RENEW_WINDOW = 20  # in seconds
@@ -53,11 +53,29 @@ class APIHarness:
                  creds: dict = None,
                  client_id: str = None, client_secret: str = None,
                  ssl_verify: bool = True, proxy: dict = None,
-                 timeout: float or tuple = None) -> object:
-        """
-        Instantiates an instance of the base class, ingests credentials,
+                 timeout: float or tuple = None,
+                 user_agent: str = None) -> object:
+        """Instantiates an instance of the class, ingests credentials,
         the base URL and the SSL verification boolean.
         Afterwards class attributes are initialized.
+
+        Keyword arguments:
+        base_url -- CrowdStrike API URL to use for requests. [Default: US-1]
+        ssl_verify -- Boolean specifying if SSL verification should be used. [Default: True]
+        proxy -- Dictionary of proxies to be used for requests.
+        timeout -- Float or tuple specifying timeouts to use for requests.
+        creds -- Dictionary containing CrowdStrike API credentials.
+                 Mutually exclusive to client_id / client_secret.
+                 {
+                     "client_id": "CLIENT_ID_HERE",
+                     "client_secret": "CLIENT_SECRET_HERE"
+                 }
+        client_id -- Client ID for the CrowdStrike API. Mutually exclusive to creds.
+        client_secret -- Client Secret for the CrowdStrike API. Mutually exclusive to creds.
+        user_agent -- User-Agent string to use for all requests made to the CrowdStrike API.
+                      String. Defaults to crowdstrike-falconpy/VERSION.
+
+        This method only accepts keywords to specify arguments.
         """
         if client_id and client_secret and not creds:
             creds = {
@@ -67,7 +85,7 @@ class APIHarness:
         elif not creds:
             creds = {}
         self.creds = creds
-        self.base_url = base_url
+        self.base_url = confirm_base_url(base_url)
         self.ssl_verify = ssl_verify
         self.proxy = proxy
         self.timeout = timeout
@@ -75,11 +93,13 @@ class APIHarness:
         self.token_expiration = 0
         self.token_time = time.time()
         self.authenticated = False
-        self.headers = lambda: {'Authorization': 'Bearer {}'.format(self.token)} if self.token else {}
+        self.headers = lambda: {"Authorization": f"Bearer {self.token}"} if self.token else {}
         self.commands = api_endpoints
+        self.user_agent = user_agent  # Issue #365
 
     def valid_cred_format(self: object) -> bool:
-        """Returns a boolean indicating if the client_id and client_secret are present in the creds dictionary."""
+        """Returns a boolean indicating if the client_id and
+        client_secret are present in the creds dictionary."""
         retval = False
         if "client_id" in self.creds and "client_secret" in self.creds:
             retval = True
@@ -95,7 +115,7 @@ class APIHarness:
         return retval
 
     def authenticate(self: object) -> bool:
-        """ Generates an authorization token. """
+        """Generates an authorization token."""
         target = self.base_url+'/oauth2/token'
         data_payload = {}
         if self.valid_cred_format():
@@ -112,7 +132,8 @@ class APIHarness:
                                  headers={},
                                  verify=self.ssl_verify,
                                  proxy=self.proxy,
-                                 timeout=self.timeout
+                                 timeout=self.timeout,
+                                 user_agent=self.user_agent
                                  )
         if result["status_code"] == 201:
             self.token = result["body"]["access_token"]
@@ -125,15 +146,15 @@ class APIHarness:
         return self.authenticated
 
     def deauthenticate(self: object) -> bool:
-        """ Revokes the specified authorization token. """
+        """Revokes the current authorization token. """
         target = str(self.base_url)+'/oauth2/revoke'
-        header_payload = {'Authorization': 'basic {}'.format(generate_b64cred(self.creds["client_id"],
-                                                                              self.creds["client_secret"]
-                                                                              ))}
-        data_payload = {'token': '{}'.format(self.token)}
+        b64cred = generate_b64cred(self.creds["client_id"], self.creds["client_secret"])
+        header_payload = {"Authorization": f"basic {b64cred}"}
+        data_payload = {"token": f"{self.token}"}
         revoked = False
         if perform_request(method="POST", endpoint=target, data=data_payload,
-                           headers=header_payload, verify=self.ssl_verify, proxy=self.proxy, timeout=self.timeout
+                           headers=header_payload, verify=self.ssl_verify,
+                           proxy=self.proxy, timeout=self.timeout, user_agent=self.user_agent
                            )["status_code"] == 200:
             self.authenticated = False
             self.token = False
@@ -144,7 +165,8 @@ class APIHarness:
         return revoked
 
     def _create_header_payload(self: object, passed_arguments: dict) -> dict:
-        """Creates the HTTP header payload based upon the existing class headers and passed arguments."""
+        """Creates the HTTP header payload based upon
+        the existing class headers and passed arguments."""
         payload = self.headers()
         if "headers" in passed_arguments:
             for item in passed_arguments["headers"]:
@@ -155,23 +177,26 @@ class APIHarness:
         return payload
 
     def command(self: object, *args, **kwargs):
-        """ Checks token expiration, renewing when necessary, then performs the request.
+        """Checks token expiration, renewing when necessary, then performs the request.
 
-            Accepted arguments (name: type = default)
-            action: str = ""                                    - API Operation to perform
-            parameters: dict = {}                               - Parameter payload (Query string)
-            body: dict = {}                                     - Body payload (Body)
-            data: dict = {}                                     - Data payload (Data)
-            headers: dict = {}                                  - Headers dictionary (HTTP Headers)
-            ids: list or str = None                             - ID list (IDs to handle)
-            partition: int or str = None                        - Partition number
-            override: str = None   (format: 'METHOD,ENDPOINT')  - Override method and endpoint
-            action_name: str = None                             - Action to perform (API specific)
-            files: list = []                                    - List of files to upload
-            file_name: str = None                               - Name of the file to upload
-            content_type: str = None                            - Content_Type HTTP header
+        Keyword arguments:
+        action: str = ""                                    - API Operation ID to perform
+        parameters: dict = {}                               - Parameter payload (Query string)
+        body: dict = {}                                     - Body payload (Body)
+        data: dict = {}                                     - Data payload (Data)
+        headers: dict = {}                                  - Headers dictionary (HTTP Headers)
+        ids: list or str = None                             - ID list (IDs to handle)
+        partition: int or str = None                        - Partition number
+        override: str = None   (format: 'METHOD,ENDPOINT')  - Override method and endpoint
+        action_name: str = None                             - Action to perform (API specific)
+        files: list = []                                    - List of files to upload
+        file_name: str = None                               - Name of the file to upload
+        content_type: str = None                            - Content_Type HTTP header
+
+        The first argument passed to this method is assumed to be 'action'. All others are ignored.
+
+        Returns: dict object containing API response or binary object depending on operation ID.
         """
-
         if self.token_expired():
             # Authenticate them if we can
             self.authenticate()
@@ -198,8 +223,10 @@ class APIHarness:
             parameter_payload = kwargs.get("parameters", {})
             # Check for authentication
             if self.authenticated:
-                selected_method = uber_command[0][1].upper()            # Which HTTP method to execute
-                if selected_method in _ALLOWED_METHODS:                 # Only accept allowed HTTP methods
+                # Which HTTP method to execute
+                selected_method = uber_command[0][1].upper()
+                # Only accept allowed HTTP methods
+                if selected_method in _ALLOWED_METHODS:
                     returned = perform_request(method=selected_method,
                                                endpoint=target,
                                                body=body_payload,
@@ -209,11 +236,13 @@ class APIHarness:
                                                files=file_list,
                                                verify=self.ssl_verify,
                                                proxy=self.proxy,
-                                               timeout=self.timeout
+                                               timeout=self.timeout,
+                                               user_agent=self.user_agent
                                                )
                 else:
                     # Bad HTTP method
-                    returned = generate_error_result(message="Invalid HTTP method specified.", code=405)
+                    returned = generate_error_result(message="Invalid HTTP method specified.",
+                                                     code=405)
             else:
                 # Invalid token / Bad creds
                 returned = generate_error_result(message="Failed to issue token.", code=401)
